@@ -22,7 +22,7 @@ mod visitor;
 ///
 /// ```rust,ignore
 /// #[derive(Yokeable)]
-/// #[yoke(manually_prove_covariance)]
+/// #[yoke(prove_covariance_manually)]
 /// ```
 ///
 /// Beyond this case, if the derive fails to compile due to lifetime issues, it
@@ -52,15 +52,19 @@ fn yokeable_derive_impl(input: &DeriveInput) -> TokenStream2 {
             // This is safe because there are no lifetime parameters.
             unsafe impl<'a, #(#tybounds),*> yoke::Yokeable<'a> for #name<#(#typarams),*> where #(#static_bounds),* {
                 type Output = Self;
+                #[inline]
                 fn transform(&self) -> &Self::Output {
                     self
                 }
+                #[inline]
                 fn transform_owned(self) -> Self::Output {
                     self
                 }
+                #[inline]
                 unsafe fn make(this: Self::Output) -> Self {
                     this
                 }
+                #[inline]
                 fn transform_mut<F>(&'a mut self, f: F)
                 where
                     F: 'static + for<'b> FnOnce(&'b mut Self::Output) {
@@ -129,6 +133,7 @@ fn yokeable_derive_impl(input: &DeriveInput) -> TokenStream2 {
                     where #(#static_bounds,)*
                     #(#yoke_bounds,)* {
                     type Output = #name<'a, #(#typarams),*>;
+                    #[inline]
                     fn transform(&'a self) -> &'a Self::Output {
                         unsafe {
                             // safety: we have asserted covariance in
@@ -136,9 +141,11 @@ fn yokeable_derive_impl(input: &DeriveInput) -> TokenStream2 {
                             ::core::mem::transmute(self)
                         }
                     }
+                    #[inline]
                     fn transform_owned(self) -> Self::Output {
                         match self { #body }
                     }
+                    #[inline]
                     unsafe fn make(this: Self::Output) -> Self {
                         use core::{mem, ptr};
                         // unfortunately Rust doesn't think `mem::transmute` is possible since it's not sure the sizes
@@ -148,6 +155,7 @@ fn yokeable_derive_impl(input: &DeriveInput) -> TokenStream2 {
                         mem::forget(this);
                         ptr::read(ptr)
                     }
+                    #[inline]
                     fn transform_mut<F>(&'a mut self, f: F)
                     where
                         F: 'static + for<'b> FnOnce(&'b mut Self::Output) {
@@ -167,12 +175,15 @@ fn yokeable_derive_impl(input: &DeriveInput) -> TokenStream2 {
             // necessary
             unsafe impl<'a, #(#tybounds),*> yoke::Yokeable<'a> for #name<'static, #(#typarams),*> where #(#static_bounds),* {
                 type Output = #name<'a, #(#typarams),*>;
+                #[inline]
                 fn transform(&'a self) -> &'a Self::Output {
                     self
                 }
+                #[inline]
                 fn transform_owned(self) -> Self::Output {
                     self
                 }
+                #[inline]
                 unsafe fn make(this: Self::Output) -> Self {
                     use core::{mem, ptr};
                     // unfortunately Rust doesn't think `mem::transmute` is possible since it's not sure the sizes
@@ -182,6 +193,7 @@ fn yokeable_derive_impl(input: &DeriveInput) -> TokenStream2 {
                     mem::forget(this);
                     ptr::read(ptr)
                 }
+                #[inline]
                 fn transform_mut<F>(&'a mut self, f: F)
                 where
                     F: 'static + for<'b> FnOnce(&'b mut Self::Output) {
@@ -202,27 +214,32 @@ fn yokeable_derive_impl(input: &DeriveInput) -> TokenStream2 {
 /// for types with a lifetime parameter.
 ///
 /// Apply the `#[yoke(cloning_zcf)]` attribute if you wish for this custom derive
-/// to use `.clone()` for its implementation.
+/// to use `.clone()` for its implementation. The attribute can be applied to
+/// fields as well.
 #[proc_macro_derive(ZeroCopyFrom, attributes(yoke))]
 pub fn zcf_derive(input: TokenStream) -> TokenStream {
     let input = parse_macro_input!(input as DeriveInput);
     TokenStream::from(zcf_derive_impl(&input))
 }
 
+fn has_cloning_zcf_attr(attrs: &[syn::Attribute]) -> bool {
+    attrs.iter().any(|a| {
+        if let Ok(i) = a.parse_args::<Ident>() {
+            if i == "cloning_zcf" {
+                return true;
+            }
+        }
+        false
+    })
+}
+
 fn zcf_derive_impl(input: &DeriveInput) -> TokenStream2 {
     let tybounds = input.generics.type_params().collect::<Vec<_>>();
     let typarams = tybounds
         .iter()
         .map(|ty| ty.ident.clone())
         .collect::<Vec<_>>();
-    let has_clone = input.attrs.iter().any(|a| {
-        if let Ok(i) = a.parse_args::<Ident>() {
-            if i == "cloning_zcf" {
-                return true;
-            }
-        }
-        false
-    });
+    let has_clone = has_cloning_zcf_attr(&input.attrs);
     let lts = input.generics.lifetimes().count();
     let name = &input.ident;
     if lts == 0 {
@@ -236,8 +253,8 @@ fn zcf_derive_impl(input: &DeriveInput) -> TokenStream2 {
             .map(|ty| parse_quote!(#ty: #clone_trait + 'static))
             .collect();
         quote! {
-            impl<#(#tybounds),*> ZeroCopyFrom<#name<#(#typarams),*>> for #name<#(#typarams),*> where #(#bounds),* {
-                fn zero_copy_from(this: &Self) -> Self {
+            impl<'zcf, #(#tybounds),*> ZeroCopyFrom<'zcf, #name<#(#typarams),*>> for #name<#(#typarams),*> where #(#bounds),* {
+                fn zero_copy_from(this: &'zcf Self) -> Self {
                     #clone
                 }
             }
@@ -252,8 +269,8 @@ fn zcf_derive_impl(input: &DeriveInput) -> TokenStream2 {
         }
         if has_clone {
             return quote! {
-                impl<'data> ZeroCopyFrom<#name<'data>> for #name<'static> {
-                    fn zero_copy_from<'b>(this: &'b #name<'data>) -> #name<'b> {
+                impl<'zcf> ZeroCopyFrom<'zcf, #name<'_>> for #name<'zcf> {
+                    fn zero_copy_from(this: &'zcf #name<'_>) -> Self {
                         this.clone()
                     }
                 }
@@ -262,49 +279,51 @@ fn zcf_derive_impl(input: &DeriveInput) -> TokenStream2 {
 
         let structure = Structure::new(input);
         let generics_env = typarams.iter().cloned().collect();
-        let static_bounds: Vec<WherePredicate> = typarams
-            .iter()
-            .map(|ty| parse_quote!(#ty: 'static))
-            .collect();
 
         let mut zcf_bounds: Vec<WherePredicate> = vec![];
         let body = structure.each_variant(|vi| {
+            let variant_cloning = has_cloning_zcf_attr(vi.ast().attrs);
             vi.construct(|f, i| {
+                let binding_cloning = variant_cloning || has_cloning_zcf_attr(&f.attrs);
                 let binding = format!("__binding_{}", i);
                 let field = Ident::new(&binding, Span::call_site());
-                let fty = replace_lifetime(&f.ty, static_lt());
-                let lifetime_ty = replace_lifetime(&f.ty, custom_lt("'data"));
 
-                let (has_ty, has_lt) = visitor::check_type_for_parameters(&f.ty, &generics_env);
-                if has_ty {
-                    // For types without type parameters, the compiler can figure out that the field implements
-                    // ZeroCopyFrom on its own. However, if there are type parameters, there may be complex preconditions
-                    // to `FieldTy: ZeroCopyFrom` that need to be satisfied. We get them to be satisfied by requiring
-                    // `FieldTy<'static>: ZeroCopyFrom<FieldTy<'data>>` as well as
-                    // `for<'data_hrtb> FieldTy<'static>: Yokeable<'data_hrtb, Output = FieldTy<'data_hrtb>>`
-                    if has_lt {
-                        let hrtb_ty = replace_lifetime(&f.ty, custom_lt("'data_hrtb"));
-                        zcf_bounds.push(parse_quote!(#fty: yoke::ZeroCopyFrom<#lifetime_ty>));
-                        zcf_bounds.push(parse_quote!(for<'data_hrtb> #fty: yoke::Yokeable<'data_hrtb, Output = #hrtb_ty>));
-                    } else {
-                        zcf_bounds.push(parse_quote!(#fty: yoke::ZeroCopyFrom<#fty>));
-                        zcf_bounds.push(parse_quote!(for<'data_hrtb> #fty: yoke::Yokeable<'data_hrtb, Output = #fty>));
+                if binding_cloning {
+                    quote! {
+                        #field.clone()
                     }
-                }
+                } else {
+                    let fty = replace_lifetime(&f.ty, custom_lt("'zcf"));
+                    let lifetime_ty = replace_lifetime(&f.ty, custom_lt("'zcf_inner"));
 
-                // By doing this we essentially require ZCF to be implemented
-                // on all fields
-                quote! {
-                    <#fty as yoke::ZeroCopyFrom<#lifetime_ty>>::zero_copy_from(#field)
+                    let (has_ty, has_lt) = visitor::check_type_for_parameters(&f.ty, &generics_env);
+                    if has_ty {
+                        // For types without type parameters, the compiler can figure out that the field implements
+                        // ZeroCopyFrom on its own. However, if there are type parameters, there may be complex preconditions
+                        // to `FieldTy: ZeroCopyFrom` that need to be satisfied. We get them to be satisfied by requiring
+                        // `FieldTy<'zcf>: ZeroCopyFrom<'zcf, FieldTy<'zcf_inner>>`
+                        if has_lt {
+                            zcf_bounds
+                                .push(parse_quote!(#fty: yoke::ZeroCopyFrom<'zcf, #lifetime_ty>));
+                        } else {
+                            zcf_bounds.push(parse_quote!(#fty: yoke::ZeroCopyFrom<'zcf, #fty>));
+                        }
+                    }
+
+                    // By doing this we essentially require ZCF to be implemented
+                    // on all fields
+                    quote! {
+                        <#fty as yoke::ZeroCopyFrom<'zcf, #lifetime_ty>>::zero_copy_from(#field)
+                    }
                 }
             })
         });
 
         quote! {
-            impl<'data, #(#tybounds),*> yoke::ZeroCopyFrom<#name<'data, #(#typarams),*>> for #name<'static, #(#typarams),*>
-                where #(#static_bounds,)*
+            impl<'zcf, 'zcf_inner, #(#tybounds),*> yoke::ZeroCopyFrom<'zcf, #name<'zcf_inner, #(#typarams),*>> for #name<'zcf, #(#typarams),*>
+                where
                 #(#zcf_bounds,)* {
-                fn zero_copy_from<'b>(this: &'b #name<'data, #(#typarams),*>) -> #name<'b, #(#typarams),*> {
+                fn zero_copy_from(this: &'zcf #name<'zcf_inner, #(#typarams),*>) -> Self {
                     match *this { #body }
                 }
             }
